from plone.app.testing.bbb import PloneTestCase


class AnalyticsTestCase(PloneTestCase):
    """ Base testing class for analytics tests """


class AnalyticsFunctionalTestCase(PloneTestCase):
    """ Base testing class for analytics functional tests """
