from plone.app.testing.bbb import PloneTestCase


class GlobalsTestCase(PloneTestCase):
    """ Base testing class for globals tests """


class GlobalsFunctionalTestCase(PloneTestCase):
    """ Base testing class for globals functional tests """
